/*
	The Battle Grounds 2 - A Source modification
	Copyright (C) 2005, The Battle Grounds 2 Team and Contributors

	The Battle Grounds 2 free software; you can redistribute it and/or
	modify it under the terms of the GNU Lesser General Public
	License as published by the Free Software Foundation; either
	version 2.1 of the License, or (at your option) any later version.

	The Battle Grounds 2 is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
	Lesser General Public License for more details.

	You should have received a copy of the GNU Lesser General Public
	License along with this library; if not, write to the Free Software
	Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

	Contact information:
		Tomas "Tjoppen" Hrdin		tjoppen@gamedev.se

	You may also contact the (future) team via the Battle Grounds website and/or forum at:
		www.bgmod.com

	 Note that because of the sheer volume of files in the Source SDK this
	notice cannot be put in all of them, but merely the ones that have any
	changes from the original SDK.
	 In order to facilitate easy searching, all changes are and must be
	commented on the following form:

	//BG2 - <name of contributer>[ - <small description>]
*/

#include "cbase.h"
#include "flag.h"
#ifdef CLIENT_DLL
	#include "c_hl2mp_player.h"
#else
	#include "hl2mp_player.h"
	#include "hl2mp_gamerules.h"
	#include "gamerules.h"
	#include "team.h"
	#include "engine/IEngineSound.h"
#endif

// memdbgon must be the last include file in a .cpp file!!!
#include "tier0/memdbgon.h"

#ifdef CLIENT_DLL
#define CFlag C_Flag
#endif

#ifndef CLIENT_DLL
void ClientPrintAll( char *str, bool printfordeadplayers = false )
{
	if( !str )
		return;

	static float flNextClientPrintAll = 0;

	if( flNextClientPrintAll > gpGlobals->curtime && !printfordeadplayers )
		return;

	flNextClientPrintAll = gpGlobals->curtime + 1.0f;

	CBasePlayer *pPlayer = NULL;

	while( (pPlayer = (CBasePlayer*)gEntList.FindEntityByClassname( pPlayer, "player" )) != NULL )
	{
		if( !pPlayer->IsAlive() && !printfordeadplayers )	//this doesn't concern dead players
			continue;

		ClientPrint( pPlayer, HUD_PRINTCENTER, str );
	}
}
#endif

class CFlag : public CBaseAnimating
{
	DECLARE_CLASS( CFlag, CBaseAnimating );
	DECLARE_NETWORKCLASS(); 
	DECLARE_PREDICTABLE();
	DECLARE_DATADESC();

	//int		m_iLastTeam;
	CNetworkVar( int, m_iLastTeam );
	float	m_flNextCapture;

	int		m_iCapturePlayers;	//how many player must be nearby to capture this flag?
	float	m_flCaptureTime,	//.. and for how long?
			m_flCaptureRadius;	//.. and how close?

	string_t	m_sFlagName;

public:

	void Spawn( void )
	{
		Precache( );
		SetModel( "models/other/flag_n.mdl" );
		//SetIdealActivity( ACT_VM_IDLE );
		//SetTouch(&CFlag::MyTouch);
		
		//SetSequence( SelectWeightedSequence( ACT_VM_IDLE ) );
		//SetSequence(0);
		
		BaseClass::Spawn( );

		SetThink( &CFlag::Think );
		SetNextThink( gpGlobals->curtime );

		m_iLastTeam = TEAM_UNASSIGNED;
		//SetSolid( SOLID_BBOX );
		//SetSolid( SOLID_BBOX );
		//m_iTeam = TEAM_UNASSIGNED;
	}
	void Precache( void )
	{
		PrecacheModel ("models/other/flag_n.mdl");
		PrecacheModel ("models/other/flag_a.mdl");
		PrecacheModel ("models/other/flag_b.mdl");

		PrecacheScriptSound( "British.win" );
		PrecacheScriptSound( "Americans.win" );
		PrecacheScriptSound( "Flag.capture" );

		//PrecacheScriptSound( "ItemBattery.Touch" );

	}
	/*void MyTouch( CBaseEntity *pEntity )
	{
#ifndef CLIENT_DLL
		CBasePlayer *pPlayer = (CBasePlayer*)pEntity;
		if( !pPlayer )
			return;
		
		if( pPlayer->GetTeamNumber() != GetTeamNumber() )
		{
			Msg( "%s touched the flag\n", pPlayer->PlayerData()->netname );
			ChangeTeam( pPlayer->GetTeamNumber() );
			CFlagHandler::Update();
		}

		//PhysicsRemoveTouchedList( this );

#endif
		/*SetCheckUntouch( true );
		EndTouch( pEntity );
		SetTouch(&CFlag::MyTouch);*//*
	}*/

	void Think( void )
	{
		//SetSequence( SelectWeightedSequence( ACT_VM_IDLE ) );

#ifndef CLIENT_DLL
		CBasePlayer *pPlayer = NULL;

		int americans = 0,
			british = 0;

		while( (pPlayer = (CBasePlayer*)gEntList.FindEntityByClassnameWithin( pPlayer, "player", GetLocalOrigin(), m_flCaptureRadius )) != NULL )
		{
			if( !pPlayer->IsAlive() )	//dead players don't cap
				continue;

			switch( pPlayer->GetTeamNumber() )
			{
			case TEAM_AMERICANS:
				americans++;
				break;
			case TEAM_BRITISH:
				british++;
				break;
			default:
				break;
			}
		}

		char *msg = NULL;

		if( americans + british > 0 && (americans <= 0 || british <= 0) )
		{
			//Msg( "\namericans = %i  british = %i\n", americans, british );

			//only americans or british at the flag
			//if we don't already own it, and we've been here for at least three seconds - capture
			if( americans >= m_iCapturePlayers && GetTeamNumber() != TEAM_AMERICANS )
			{
				if( m_iLastTeam != TEAM_AMERICANS )
				{
					char msg2[512];
					//Msg( "americans are capturing a flag(\"%s\")\n", STRING( m_sFlagName ) );
					Q_snprintf( msg2, 512, "The americans are capturing a flag(%s)", STRING( m_sFlagName ) );
					msg = msg2;
					m_iLastTeam = TEAM_AMERICANS;
					m_flNextCapture = gpGlobals->curtime + m_flCaptureTime;
				}
				else if( gpGlobals->curtime >= m_flNextCapture )
				{
					char msg2[512];
                    //CFlagHandler::PlayCaptureSound();
					Q_snprintf( msg2, 512, "The americans captured a flag(%s)", STRING( m_sFlagName ) );
					msg = msg2;
					EmitSound( "Flag.capture" );
					ChangeTeam( TEAM_AMERICANS );
					CFlagHandler::Update();
				}
			}
			else if( british >= m_iCapturePlayers && GetTeamNumber() != TEAM_BRITISH )
			{
				if( m_iLastTeam != TEAM_BRITISH )
				{
					char msg2[512];
                    //Msg( "british are capturing a flag(\"%s\")\n", STRING( m_sFlagName ) );
					Q_snprintf( msg2, 512, "The british are capturing a flag(%s)", STRING( m_sFlagName ) );
					msg = msg2;
					m_iLastTeam = TEAM_BRITISH;
					m_flNextCapture = gpGlobals->curtime + m_flCaptureTime;
				}
				else if( gpGlobals->curtime >= m_flNextCapture )
				{
					char msg2[512];
					//CFlagHandler::PlayCaptureSound();
					Q_snprintf( msg2, 512, "The british captured a flag(%s)", STRING( m_sFlagName ) );
					msg = msg2;
					EmitSound( "Flag.capture" );
					ChangeTeam( TEAM_BRITISH );
					CFlagHandler::Update();
				}
			}
		}
		else
		{
			if( m_iLastTeam != TEAM_UNASSIGNED )
			{
				//Msg( "stopped capturing a flag\n" );
				if( m_iLastTeam == TEAM_AMERICANS && GetTeamNumber() != TEAM_AMERICANS )
				{
					char msg2[512];
					Q_snprintf( msg2, 512, "The americans stopped capturing a flag(%s)", STRING( m_sFlagName ) );
					msg = msg2;
				}
				else if( m_iLastTeam == TEAM_BRITISH && GetTeamNumber() != TEAM_BRITISH )
				{
					char msg2[512];
					Q_snprintf( msg2, 512, "The british stopped capturing a flag(%s)", STRING( m_sFlagName ) );
					msg = msg2;
				}
			}
			//noone here
			m_iLastTeam = TEAM_UNASSIGNED;
		}

		ClientPrintAll( msg );
#endif
		SetNextThink( gpGlobals->curtime + 0.5f );
	}

	void ChangeTeam( int iTeamNum )
	{
#ifndef CLIENT_DLL
		switch( iTeamNum )
		{
		case TEAM_AMERICANS:
			SetModel( "models/other/flag_a.mdl" );
			break;
		case TEAM_BRITISH:
			SetModel( "models/other/flag_b.mdl" );
			break;
		default:
			SetModel( "models/other/flag_n.mdl" );
			break;
		}
#endif
		BaseClass::ChangeTeam( iTeamNum );
	}
};

IMPLEMENT_NETWORKCLASS_ALIASED( Flag, DT_Flag )

BEGIN_NETWORK_TABLE( CFlag, DT_Flag )
#ifdef CLIENT_DLL
	/*RecvPropBool( RECVINFO( m_bNeedPump ) ),
	RecvPropBool( RECVINFO( m_bDelayedFire1 ) ),
	RecvPropBool( RECVINFO( m_bDelayedFire2 ) ),*/
	RecvPropInt( RECVINFO( m_iLastTeam ) ),
#else
	/*SendPropBool( SENDINFO( m_bNeedPump ) ),
	SendPropBool( SENDINFO( m_bDelayedFire1 ) ),
	SendPropBool( SENDINFO( m_bDelayedFire2 ) ),*/
	SendPropInt( SENDINFO( m_iLastTeam ) , Q_log2(NUM_TEAMS), SPROP_UNSIGNED ),
#endif
END_NETWORK_TABLE()

BEGIN_PREDICTION_DATA( CFlag )
END_PREDICTION_DATA()

BEGIN_DATADESC( CFlag )

	DEFINE_KEYFIELD( m_iCapturePlayers, FIELD_INTEGER, "CapturePlayers" ),
	DEFINE_KEYFIELD( m_flCaptureRadius, FIELD_FLOAT, "CaptureRadius" ),
	DEFINE_KEYFIELD( m_flCaptureTime, FIELD_FLOAT, "CaptureTime" ),

	DEFINE_KEYFIELD( m_sFlagName, FIELD_STRING, "FlagName" ),

END_DATADESC()

LINK_ENTITY_TO_CLASS(flag, CFlag);
PRECACHE_REGISTER(flag);

#ifndef CLIENT_DLL
void CFlagHandler::RespawnAll( char *pSound )
{
	int x;
	for( x = 0; x < g_Teams[TEAM_AMERICANS]->GetNumPlayers(); x++ )
	{
		CBasePlayer *pPlayer = g_Teams[TEAM_AMERICANS]->GetPlayer( x );
		/*if( pPlayer->IsAlive() )
			pPlayer->Kill();*/
		pPlayer->Spawn();

		if( pSound )
			pPlayer->EmitSound( pSound );
	}

	for( x = 0; x < g_Teams[TEAM_BRITISH]->GetNumPlayers(); x++ )
	{
		CBasePlayer *pPlayer = g_Teams[TEAM_BRITISH]->GetPlayer( x );
		/*if( pPlayer->IsAlive() )
			pPlayer->Kill();*/
		pPlayer->Spawn();

		if( pSound )
			pPlayer->EmitSound( pSound );
	}
}

/*void CFlagHandler::PlayCaptureSound( void )
{
	/*CBasePlayer *pPlayer = NULL;
	while( (pPlayer = (CBasePlayer*)gEntList.FindEntityByClassname( pPlayer, "player" )) != NULL )
		pPlayer->EmitSound( "Flag.capture" );*//*
}*/

void CFlagHandler::ResetFlags( void )
{
	CBaseEntity *pEntity = NULL;
	
	while( (pEntity = gEntList.FindEntityByClassname( pEntity, "flag" )) != NULL )
	{
		CFlag *pFlag = (CFlag*)pEntity;
		if( !pFlag )
			continue;

		pFlag->ChangeTeam( TEAM_UNASSIGNED );
	}
}

void CFlagHandler::Update( void )
{
	CBaseEntity *pEntity = NULL;
	
	int	american_flags = 0,
		british_flags = 0,
		neutral_flags = 0;

	while( (pEntity = gEntList.FindEntityByClassname( pEntity, "flag" )) != NULL )
	{
		CFlag *pFlag = (CFlag*)pEntity;
		if( !pFlag )
			continue;
		
		switch( pFlag->GetTeamNumber() )
		{
		case TEAM_AMERICANS:
			american_flags++;
			break;
		case TEAM_BRITISH:
			british_flags++;
			break;
		default:
			neutral_flags++;
			break;
		}
	}

	/*Msg( "american_flags = %i\n", american_flags );
	Msg( "british_flags = %i\n", british_flags );
	Msg( "neutral_flags = %i\n", neutral_flags );*/

	if( !american_flags && !british_flags && !neutral_flags )
		return;

	if( neutral_flags > 0 )
		return;

	if( american_flags <= 0 && british_flags <= 0 )
	{
		//draw
		//Msg( "draw\n" );
		ClientPrintAll( "Draw", true );
		ResetFlags();
		RespawnAll( NULL );
		return;
	}

	if( american_flags <= 0 )
	{
		//british win
		//Msg( "british win\n" );
		ClientPrintAll( "The british win", true );
		g_Teams[TEAM_BRITISH]->AddScore( 1 );
		ResetFlags();
		RespawnAll( "British.win" );
		return;
	}

	if( british_flags <= 0 )
	{
		//americans win
		//Msg( "americans win\n" );
		ClientPrintAll( "The americans win", true );
		g_Teams[TEAM_AMERICANS]->AddScore( 1 );
		ResetFlags();
		RespawnAll( "Americans.win" );
		return;
	}
}
#endif