/*
	The Battle Grounds 2 - A Source modification
	Copyright (C) 2005, The Battle Grounds 2 Team and Contributors

	The Battle Grounds 2 free software; you can redistribute it and/or
	modify it under the terms of the GNU Lesser General Public
	License as published by the Free Software Foundation; either
	version 2.1 of the License, or (at your option) any later version.

	The Battle Grounds 2 is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
	Lesser General Public License for more details.

	You should have received a copy of the GNU Lesser General Public
	License along with this library; if not, write to the Free Software
	Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

	Contact information:
		Tomas "Tjoppen" Hrdin		tjoppen@gamedev.se

	You may also contact the (future) team via the Battle Grounds website and/or forum at:
		www.bgmod.com

	 Note that because of the sheer volume of files in the Source SDK this
	notice cannot be put in all of them, but merely the ones that have any
	changes from the original SDK.
	 In order to facilitate easy searching, all changes are and must be
	commented on the following form:

	//BG2 - <name of contributer>[ - <small description>]
*/

#include "cbase.h"
#include "npcevent.h"
#include "in_buttons.h"

#ifdef CLIENT_DLL
	#include "c_hl2mp_player.h"
#else
	#include "hl2mp_player.h"
	#include "te_effect_dispatch.h"
#endif

#include "weapon_hl2mpbasehlmpcombatweapon.h"
#include "weapon_hl2mpbasebasebludgeon.h"

#define BLUDGEON_HULL_DIM		16

static const Vector g_bludgeonMins(-BLUDGEON_HULL_DIM,-BLUDGEON_HULL_DIM,-BLUDGEON_HULL_DIM);
static const Vector g_bludgeonMaxs(BLUDGEON_HULL_DIM,BLUDGEON_HULL_DIM,BLUDGEON_HULL_DIM);

#ifdef CLIENT_DLL
#define CBaseBG2Weapon C_BaseBG2Weapon
#endif

class CBaseBG2Weapon : public CBaseHL2MPCombatWeapon
{
	DECLARE_CLASS( CBaseBG2Weapon, CBaseHL2MPCombatWeapon );
public:
	enum
	{
		ATTACKTYPE_NONE = 0,
		ATTACKTYPE_STAB,
		ATTACKTYPE_SLASH,
		ATTACKTYPE_FIREARM,
		/*
		other attacktypes might be
		ATTACKTYPE_THROW,

		*/
	};

	enum
	{
		ATTACK_NONE = -1,		//this enum used for separating different attacks and stuff
		ATTACK_PRIMARY = 0,
		ATTACK_SECONDARY = 1,
	};

	struct attackinfo
	{
		int		m_iAttacktype;
		Activity	m_iAttackActivity;

		float	m_flRange,
				m_flDamage,
				m_flRecoil,				//factor of 357 standard recoil - firearm only
				m_flCosAngleTolerance,	//tolerance of melee hits(sqrt(0.5) for crowbar, or 45 degrees)
										// - melee only
				m_flAttackrate;

		Vector	m_vDuckSpread,			//firearm only
				m_vStandSpread;			//firearm only
	};

	attackinfo	m_Attackinfos[2];

	/*float		m_flRange,
				m_flFireRate,
				m_flImpactDelay,
				m_flShotDamage,
				m_flStabDamage;

	Vector		m_vDuckSpread,
				m_vStandSpread;*/

	CBaseBG2Weapon( void );

	void	PrimaryAttack( void );
	void	SecondaryAttack( void );

	int		GetAttackType( int iAttack )
	{
		return iAttack != ATTACK_NONE ? m_Attackinfos[iAttack].m_iAttacktype : ATTACK_NONE;
	}

	float	GetRange( int iAttack )
	{
		return iAttack != ATTACK_NONE ? m_Attackinfos[iAttack].m_flRange : 0;
	}

	float	GetAttackRate( int iAttack )
	{
		if( iAttack == ATTACK_NONE )
			return 0;

		if( m_Attackinfos[iAttack].m_flAttackrate < 0 )
			return SequenceDuration();

		return m_Attackinfos[iAttack].m_flAttackrate;
	}

	float	GetRecoil( int iAttack )
	{
		if( iAttack == ATTACK_NONE )
			return 0;

		if( GetAttackType(iAttack) == ATTACKTYPE_STAB )
			return 0.5f;	//stab weapons all have the same recoil...

		if( GetAttackType(iAttack) == ATTACKTYPE_SLASH )
			return 0.5f;

		return m_Attackinfos[iAttack].m_flRecoil;
	}

	float	GetDamage( int iAttack )
	{
		if( iAttack == ATTACK_NONE )
			return 0;

		float scale = GetAttackType( iAttack ) == ATTACKTYPE_STAB ? 0.8f : 1.0f;	//adjusting stabs

		return m_Attackinfos[iAttack].m_flDamage * scale
					* random->RandomFloat( 0.97f, 1.03f );	//+-3% dmg
	}

	Vector	GetSpread( int iAttack )
	{
		if( iAttack == ATTACK_NONE )
			return vec3_origin;

		bool moving = false;
		if( GetOwner() && GetOwner()->GetLocalVelocity().Length() > 50.0f )
			moving = true;	//moving fast enough to increase spread..

		if( GetOwner() && (GetOwner()->GetFlags() & FL_DUCKING) )
			return m_Attackinfos[iAttack].m_vDuckSpread * (moving ? 2.0f : 1.0f);

		return m_Attackinfos[iAttack].m_vStandSpread * (moving ? 2.0f : 1.0f);
	}

	Activity	GetActivity( int iAttack )
	{
		return iAttack != ATTACK_NONE ? m_Attackinfos[iAttack].m_iAttackActivity : (Activity)0;
	}

	float	GetCosTolerance( int iAttack )
	{
		return iAttack != ATTACK_NONE ? m_Attackinfos[iAttack].m_flCosAngleTolerance : 0;
	}

	/*bool	IsMeleeWeapon( void )
	{
		return m_Attackinfos[ATTACK_PRIMARY].m_iAttacktype == ATTACKTYPE_STAB ? true : false;
	}*/

	//void		ItemPostFrame( void );
	void		Fire( int iAttack );
	void		Swing( int iAttack );

	void		Hit( trace_t &traceHit, int iAttack );
	Activity	ChooseIntersectionPointAndActivity( int iAttack, trace_t &hitTrace, const Vector &mins, const Vector &maxs, CBasePlayer *pOwner );
	bool		ImpactWater( const Vector &start, const Vector &end );
	void		ImpactEffect( trace_t &traceHit );

	void		Drop( const Vector &vecVelocity );

	DECLARE_NETWORKCLASS(); 
	DECLARE_PREDICTABLE();

#ifndef CLIENT_DLL
	DECLARE_ACTTABLE();
#endif

private:
	CBaseBG2Weapon( const CBaseBG2Weapon & );
};


//these things must go before the macro. appropriately named of course(CWeapon##name C_Weapon##name
//the reason for this is that #define's can't be inside other #define's
//#ifdef CLIENT_DLL
//#define CWeaponRevolutionnaire C_WeaponRevolutionnaire
//#endif
#define DECLARE_BG2_WEAPON( name )								\
class CWeapon##name : public CBaseBG2Weapon {					\
	DECLARE_CLASS( CWeapon##name, CBaseBG2Weapon );				\
public:															\
	CWeapon##name( void );										\
	DECLARE_NETWORKCLASS();										\
	DECLARE_PREDICTABLE();										\
private:														\
	CWeapon##name( const CWeapon##name & );						\
};																\
IMPLEMENT_NETWORKCLASS_ALIASED( Weapon##name, DT_Weapon##name )	\
BEGIN_NETWORK_TABLE( CWeapon##name, DT_Weapon##name )			\
END_NETWORK_TABLE()												\
BEGIN_PREDICTION_DATA( CWeapon##name )							\
END_PREDICTION_DATA()											\
LINK_ENTITY_TO_CLASS( weapon_##name, CWeapon##name );			\
PRECACHE_WEAPON_REGISTER( weapon_##name );						\
CWeapon##name::CWeapon##name( void )							//constructor follows
